unit KOLSPCDirectoryEditBox;

interface
{$R *.res}
uses
  Windows, Messages, KOL;

type

  PSPCDirectoryEdit =^TSPCDirectoryEdit;
  TSPCDirectoryEditBox = PSPCDirectoryEdit;
  TSPCDirectoryEdit = object(TObj)
  private
    { Private declarations }
    fCreated: Boolean;
    fControl : PControl;
    fEdit: PControl;
    fButton: PControl;
    fDirList: POpenDirDialog;
    fFont: PGraphicTool;
    fPath: string;
    fTitle: string;
    fCaptionEmpty: string;
    fOnDirChanged: TOnEvent;
    fColor: TColor;
    procedure DoClick(Sender: PObj);
    procedure SetPath(Value: string);
    procedure DoDirChange(Obj: PObj);
  protected
    { Protected declarations }
  public
    destructor Destroy; virtual;
    procedure Initialize;
    function SetAlign(Value: TControlAlign): PSPCDirectoryEdit; overload;
    function SetPosition(X, Y: integer): PSPCDirectoryEdit; overload;
    function SetSize(X, Y: integer): PSPCDirectoryEdit; overload;
    function GetFont: PGraphicTool;
    { Public declarations }
    property Font: PGraphicTool read GetFont;
    property Color: TColor read fColor write fColor;
    property Title: string read fTitle write fTitle;
    property Path: string read fPath write SetPath;
    property OnDirChanged: TOnEvent read fOnDirChanged write fOnDirChanged;
    property CaptionEmpty: string read fCaptionEmpty write fCaptionEmpty;
  end;


function NewTSPCDirectoryEditBox(AOwner: PControl; EdgeStyle: TEdgeStyle; Bars: TScrollerBars): PSPCDirectoryEdit;

implementation

uses objects;

function NewTSPCDirectoryEditBox;
var p: PSPCDirectoryEdit;
    c: PControl;
begin
   c:=NewScrollBox(AOwner, esLowered, []);
   New(p, create);
   AOwner.Add2AutoFree(p);
   p.fControl  := c;
   p.fFont:=NewFont;
   p.fCreated:=False;
   p.OnDirChanged:=p.DoDirChange;
   Result:= p;
end;

function TSPCDirectoryEdit.SetAlign(Value: TControlAlign): PSPCDirectoryEdit;
begin
  fControl.Align:=Value;
  Result:=@Self;
end;

destructor TSPCDirectoryEdit.Destroy;
begin
   fFont.Free;
   inherited;
end;

function TSPCDirectoryEdit.SetPosition(X, Y: integer): PSPCDirectoryEdit;
begin
   fControl.Left := X;
   fControl.Top := Y;
   Result := @self;
end;

function TSPCDirectoryEdit.SetSize(X, Y: integer): PSPCDirectoryEdit;
begin
   fControl.Width  := X;
   fControl.Height := Y;
   Result := @self;
end;

function TSPCDirectoryEdit.GetFont;
begin
   Result := fFont;
end;

procedure TSPCDirectoryEdit.Initialize;
begin
   fEdit:=NewEditBox(fControl,[eoReadOnly]);
   fEdit.Font.FontHeight:=-11;
   fControl.Height:=fEdit.Height-1;
   fEdit.Left:=0;
   fEdit.Top:=0;
   fEdit.Height:=17;
   fEdit.Width:=fControl.Width-21;
   fEdit.HasBorder:=False;
   fEdit.Color:=fColor;
   fEdit.Font.Assign(Font);
   fButton:=NewBitBtn(fControl,'...',[],glyphLeft,LoadBitmap(hInstance,'DIROPENICON'),1);
   fButton.Font.FontHeight:=-11;
   fButton.LikeSpeedButton;
   fButton.Width:=17;
   fButton.Height:=17;
   fButton.Top:=0;
   fButton.Left:=fEdit.Width;
   fButton.OnClick:=DoClick;
   fDirList:=NewOpenDirDialog(Title,[]);
   fDirList.CenterOnScreen:=True;
end;

procedure TSPCDirectoryEdit.SetPath(Value: string);
begin
  if DirectoryExists(Value) then fPath:=Value else fPath:='';
  if Length(fPath)=0 then fEdit.Text:=CaptionEmpty else fEdit.Text:=fPath;
  if Assigned(fOnDirChanged) then if fCreated then fOnDirChanged(@Self) else fCreated:=True;
end;

procedure TSPCDirectoryEdit.DoClick;
begin
  fDirList.InitialPath:=Path;
  if fDirList.Execute then
  begin
    Path:=fDirList.Path;
    fEdit.Text:=fDirList.Path;
  end;
end;

procedure TSPCDirectoryEdit.DoDirChange;
begin
  if Assigned(fOnDirChanged) then fOnDirChanged(@Self);
end;

end.
