unit KOLSPCDirectoryListBox;

interface
uses
  Windows, Messages, KOL, KOLSPCFileListBox;
{$R *.res}
type

  PSPCDirectoryList =^TSPCDirectoryList;
  TSPCDirectoryListBox = PSPCDirectoryList;
  TSPCDirectoryList = object(TObj)
  private
    { Private declarations }
    fPathChanged: Boolean;
    fColor: TColor;
    fDoIndent: Boolean;
    fTotalTree: Integer;
    fIcons: PImageList;
    fIconsBmp: PBitmap;
    fInitialized: Integer;
    fCreated: Boolean;
    fIntegralHeight: Boolean;
    fDirList: PDirList;
    fCurIndex: Integer;
    fControl : PControl;
    fPath: string;
    fFont: PGraphicTool;
    FOnMouseDblClick: TOnMouse;
    fLVBkColor: Integer;
    fOnChange: TOnEvent;
    procedure DoMouseDblClick(Sender: PControl; var Mouse: TMouseEventData);
    procedure LVPaint;
    procedure SetPath(Value: string);
  protected
    { Protected declarations }
  public
    destructor Destroy; virtual;
    function SetAlign(Value: TControlAlign): PSPCDirectoryList; overload;
    function SetPosition(X, Y: integer): PSPCDirectoryList; overload;
    function SetSize(X, Y: integer): PSPCDirectoryList; overload;
    function GetFont: PGraphicTool;
    property Color: TColor read fColor write fColor;
    { Public declarations }
    property Font: PGraphicTool read GetFont;
    property IntegralHeight: Boolean read fIntegralHeight write fIntegralHeight;
    property Path: string read fPath write SetPath;
    property DoIndent: Boolean read fDoIndent write fDoIndent;
    property OnMouseDblClk: TOnMouse read FOnMouseDblClick write FOnMouseDblClick;
    property CurIndex: Integer read fCurIndex write fCurIndex;
    property LVBkColor: Integer read fLVBkColor write fLVBkColor;
    property OnChange: TOnEvent read fOnChange write fOnChange;
  end;

function NewTSPCDirectoryListBox(AOwner: PControl; Style: TListViewStyle; Options: TListViewOptions; ImageListSmall, ImageListNormal, ImageListState: PImageList): PSPCDirectoryList;

implementation

uses objects;

function NewTSPCDirectoryListBox;
var p: PSPCDirectoryList;
    c: PControl;
begin
   c:=NewListView(AOwner, lvsDetailNoHeader, [] , ImageListSmall, ImageListNormal, ImageListState);
   New(p, create);
   AOwner.Add2AutoFree(p);
   p.fControl  := c;
   p.fControl.OnMouseDblClk:=p.DoMouseDblClick;
   p.fControl.lvOptions:=[lvoRowSelect,lvoInfoTip,lvoAutoArrange];
   p.fCreated:=False;
   p.fIntegralHeight:=True;
   p.fDirList:=NewDirList('','',0);
   p.fFont:= NewFont;
   p.fIconsBmp:=NewBitmap(1,1);
   p.fIconsBmp.LoadFromResourceName(hInstance,'ICONS');
   p.fIcons:=NewImageList(AOwner);
   p.fIcons.ImgWidth:=16;
   p.fIcons.ImgHeight:=16;
   p.fIcons.ShareImages:=True;
   p.fIcons.AddMasked(p.fIconsBmp.Handle,$FFFFFF);
   p.fControl.ImageListSmall:=p.fIcons;
   p.fInitialized:=0;
   Result:= p;
end;

function TSPCDirectoryList.SetAlign(Value: TControlAlign): PSPCDirectoryList;
begin
  fControl.Align:=Value;
  Result:=@Self;
end;

procedure TSPCDirectoryList.DoMouseDblClick;
var
  s: string;
  i: Integer;
begin
  if fControl.lvCurItem>-1 then
  begin
    s:='';
    if fControl.LVCurItem<=fTotalTree-1 then
    begin
      for i:=0 to fControl.LVCurItem do s:=s+fControl.lvItems[i,0]+'\';
    end else
    begin
      for i:=0 to fTotalTree-1 do s:=s+fControl.lvItems[i,0]+'\';
      s:=s+fControl.lvItems[fControl.lvCurItem,0];
    end;
    Path:=s;
    if Assigned(fOnMouseDblClick) then fOnMouseDblClick(@Self,Mouse);
  end;
end;

destructor TSPCDirectoryList.Destroy;
begin
   fFont.Free;
   inherited;
end;

function TSPCDirectoryList.SetPosition(X, Y: integer): PSPCDirectoryList;
begin
   fControl.Left := X;
   fControl.Top := Y;
   Result := @self;
end;

function TSPCDirectoryList.SetSize(X, Y: integer): PSPCDirectoryList;
begin
   fControl.Width  := X;
   fControl.Height := Y;
   Result := @self;
end;

function TSPCDirectoryList.GetFont;
begin
   Result := fFont;
end;

procedure TSPCDirectoryList.LVPaint;
begin
  fControl.Font.Assign(Font);
end;

procedure TSPCDirectoryList.SetPath(Value: string);
var
  TPath,fValue: string;
  i,z: Integer;
  LastDir: Cardinal;
  fDriveType: Integer;
  fImgIndex: Integer;
  Code: Cardinal;
  Rect: TRect;
  MaxWidth: Integer;
  ZZZ: TMouseEventData;
begin
  fValue:=Value;
  fControl.lvBkColor:=fColor;
  fControl.lvTextBkColor:=fColor;
  if Length(fValue)=1 then fValue:=fValue+':\';
  if not fCreated then
  begin
    fCreated:=True;
    fControl.LVColAdd('',taLeft,fControl.Width-20);
    if fIntegralHeight then
    begin
      fControl.Height:=(fControl.Height div 16)*16+1;
    end;
  end;
  fControl.Clear;
  if DirectoryExists(fValue) then
  begin
    LastDir:=0;
    fTotalTree:=0;
    if fValue[Length(fValue)]='\' then TPath:=fValue else TPath:=fValue+'\';
    fPath:=TPath;
    fDriveType:=GetDriveType(PChar(Copy(fValue,1,2)));
    case fDriveType of
    0:fImgIndex:=2;
    1:fImgIndex:=2;
    2:fImgIndex:=3;
    3:fImgIndex:=2;
    4:fImgIndex:=5;
    5:fImgIndex:=4;
    else fImgIndex:=2;
    end;
    repeat
      if fTotalTree>0 then fImgIndex:=1;
      fControl.LVAdd(Copy(TPath,1,Pos('\',TPath)-1),fImgIndex,[],0,0,0);
      fControl.LVItemIndent[LastDir]:=LastDir;
      Delete(TPath,1,Pos('\',TPath));
      if DoIndent then Inc(LastDir);
      Inc(fTotalTree);
    until Length(TPath)=0;
    fDirList.ScanDirectory(fValue,'*.*',FILE_ATTRIBUTE_NORMAL);
    fDirList.Sort([sdrByName]);
    z:=-1;
    for i:=0 to fDirList.Count-1 do
    begin
      Code:=fDirList.Items[i].dwFileAttributes;
      if Code=(Code or $10) then
      if not (fDirList.Names[i]='.') then
      if not (fDirList.Names[i]='..') then
      begin
        Inc(z);
        fControl.LVAdd(fDirList.Names[i],0,[],0,0,0);
        if DoIndent then fControl.LVItemIndent[z+fTotalTree]:=LastDir else fControl.LVItemIndent[z+fTotalTree]:=1;
      end;
    end;
  end else fPath:='';
  Inc(fInitialized);
  if fInitialized>2 then fInitialized:=2;
  if Assigned(OnChange) then if fInitialized=2 then OnChange(@Self);
  fControl.LVColWidth[0]:=fControl.Width-20;
end;

end.
