unit KOLSPCFileListBox;

interface
uses
  Windows, Messages, KOL;

const
  ctDefault=0;
  ctLower=1;
  ctUpper=2;

type

  PSPCFileList =^TSPCFileList;
  TSPCFileListBox = PSPCFileList;
  TSPCFileList = object(TObj)
  private
    { Private declarations }
    fColor: TColor;
    fTotalTree: Integer;
    fMask: string;
    fIntegralHeight: Boolean;
    fDirList: PDirList;
    fCurIndex: Integer;
    fControl : PControl;
    fPath: string;
    fFont: PGraphicTool;
    FOnSelChange: TOnEvent;
    fDoCase: Integer;
    procedure DoSelChange(Sender: PObj);
    procedure LVPaint;
    procedure SetPath(Value: string);
    procedure SetMask(Value: string);
    procedure SetIntegralHeight(Value: Boolean);
    function GetCurIndex: Integer;
    procedure SetCurIndex(Value: Integer);
  protected
    { Protected declarations }
  public
    destructor Destroy; virtual;
    function GetFileName: string;
    function SetPosition(X, Y: integer): PSPCFileList; overload;
    function SetSize(X, Y: integer): PSPCFileList; overload;
    function SetAlign(Value: TControlAlign): PSPCFileList; overload;
    function GetFont: PGraphicTool;
    { Public declarations }
    property Color: TColor read fColor write fColor;
    property Font: PGraphicTool read GetFont;
    property IntegralHeight: Boolean read fIntegralHeight write SetIntegralHeight;
    property Path: string read fPath write SetPath;
    property Mask: string read fMask write SetMask;
    property OnSelChange: TOnEvent read FOnSelChange write FOnSelChange;
    property CurIndex: Integer read GetCurIndex write SetCurIndex;
    function Count: LongInt;
    property DoCase: Integer read fDoCase write fDoCase;
  end;

function NewTSPCFileListBox(AOwner: PControl; Options: TListOptions): PSPCFileList;

implementation

uses objects;

function NewTSPCFileListBox;
var p: PSPCFileList;
    c: PControl;
begin
   c:=NewListBox(AOwner,[loNoIntegralHeight]);
   New(p, create);
   AOwner.Add2AutoFree(p);
   p.fControl  := c;
   p.fControl.OnChange:=p.DoSelChange;
   p.fControl.Font.FontHeight:=-11;
   p.fDirList:=NewDirList('','',0);
   p.fFont:= NewFont;
   Result:= p;
end;

function TSPCFileList.SetAlign(Value: TControlAlign): PSPCFileList;
begin
  fControl.Align:=Value;
  Result:=@Self;
end;

procedure TSPCFileList.SetMask(Value: string);
begin
  fMask:=Value;
  Path:=Path;
end;

procedure TSPCFileList.DoSelChange;
begin
  if Assigned(fOnSelChange) then fOnSelChange(@Self);
end;

destructor TSPCFileList.Destroy;
begin
   fFont.Free;
   inherited;
end;

function TSPCFileList.SetPosition(X, Y: integer): PSPCFileList;
begin
   fControl.Left := X;
   fControl.Top := Y;
   Result := @self;
end;

function TSPCFileList.SetSize(X, Y: integer): PSPCFileList;
begin
   fControl.Width  := X;
   fControl.Height := Y;
   Result := @self;
end;

function TSPCFileList.GetFont;
begin
   Result := fFont;
end;

procedure TSPCFileList.LVPaint;
begin
  fControl.Font.Assign(Font);
end;

procedure TSPCFileList.SetPath(Value: string);
var
  TPath: string;
  i,z: Integer;
  LastDir: Cardinal;
  Code: Cardinal;
begin
  if DirectoryExists(Value) then
  begin
    fPath:=Value;
    fControl.Clear;
    fControl.Color:=fColor;
    fControl.AddDirList(Value+'\'+Mask,$FF-FILE_ATTRIBUTE_DIRECTORY);
    if fDoCase=ctLower then for i:=0 to fControl.Count-1 do fControl.Items[i]:=LowerCase(fControl.Items[i]);
    if fDoCase=ctUpper then for i:=0 to fControl.Count-1 do fControl.Items[i]:=UpperCase(fControl.Items[i]);
  end else fPath:='';
end;

procedure TSPCFileList.SetIntegralHeight;
begin
  fIntegralHeight:=Value;
  if fIntegralHeight then
  begin
    fControl.Height:=(fControl.Height div 14)*14+6;
  end;
end;

function TSPCFileList.GetFileName: string;
begin
  Result:=fControl.Items[fControl.CurIndex];
end;

function TSPCFileList.Count: LongInt;
begin
  Result:=fControl.Count;
end;

function TSPCFileList.GetCurIndex: Integer;
begin
  Result:=fControl.CurIndex;
end;

procedure TSPCFileList.SetCurIndex(Value: Integer);
begin
  fControl.CurIndex:=Value;
end;

end.

