unit KOLSPCFilterComboBox;

interface
uses
  Windows, Messages, KOL;

type

  TFilterItem=class
  private
    fFull: string;
    fDescription: string;
    fFilter: string;
  public
  published
    property Full: string read fFull write fFull;
    property Description: string read fDescription write fDescription;
    property Filter: string read fFilter write fFilter;
  end;

  PSPCFilterCombo =^TSPCFilterCombo;
  TSPCFilterComboBox = PSPCFilterCombo;
  TSPCFilterCombo = object(TObj)
  private
    { Private declarations }
    fColor: TColor;
    fCurIndex: Integer;
    fControl : PControl;
    fFont: PGraphicTool;
    fLVBkColor: Integer;
    fOnChange: TOnEvent;
    fFilterItems: PList;
    fFilter: string;
    fFilters: string;
    fCreated: Boolean;
    fInitialized: Integer;
    function GetFilterItem(Index: Integer): TFilterItem;
    procedure SetFilterItem(Index: Integer; Value: TFilterItem);
    procedure SetFilter(Value: string);
    procedure SetCurIndex(Value: Integer);
    procedure DoChange(Obj: PObj);
  protected
    { Protected declarations }
  public
    destructor Destroy; virtual;
    procedure Add;
    procedure DeleteItem(Index: Integer);
    function Count: Integer;
    procedure BuildList;
    function SetAlign(Value: TControlAlign): PSPCFilterCombo; overload;
    function SetPosition(X, Y: integer): PSPCFilterCombo; overload;
    function SetSize(X, Y: integer): PSPCFilterCombo; overload;
    function GetFont: PGraphicTool;
    property Filter: string read fFilter write SetFilter;
    property Color: TColor read fColor write fColor;
    { Public declarations }
    property Text: string read fFilters write fFilters;
    property Font: PGraphicTool read GetFont;
    property CurIndex: Integer read fCurIndex write SetCurIndex;
    property LVBkColor: Integer read fLVBkColor write fLVBkColor;
    property OnChange: TOnEvent read fOnChange write fOnChange;
    property Items[Index: Integer]: TFilterItem read GetFilterItem write SetFilterItem;
  end;

function NewTSPCFilterComboBox(AOwner: PControl; Options: TComboOptions): PSPCFilterCombo;
function GetLastPos(c: char; s: string): Integer;
function CheckBit(Value,Index: LongInt): Boolean;

implementation

uses objects;

function CheckBit;
var
  fL: LongInt;
begin
  fL:=Value;
  fL:=fL shr Index;
  fL:=fL and $01;
  if fL=1 then Result:=True else Result:=False;
end;

function GetLastPos(c: char; s: string): Integer;
var
  i: Integer;
begin
  Result:=0;
  for i:=1 to Length(s) do if s[i]=c then Result:=i;
end;

function NewTSPCFilterComboBox;
var p: PSPCFilterCombo;
    c: PControl;
    i: Integer;
    fT: string;
begin
   c:=NewComboBox(AOwner,[coReadOnly]);
   New(p,create);
   AOwner.Add2AutoFree(p);
   p.fControl:= c;
   p.fFont:=NewFont;
   p.fControl.OnChange:=p.DoChange;
   p.fFilterItems:=NewList;
   p.fCreated:=False;
   p.fInitialized:=0;
   Result:=p;
end;

function TSPCFilterCombo.SetAlign(Value: TControlAlign): PSPCFilterCombo;
begin
  fControl.Align:=Value;
  Result:=@Self;
end;

procedure TSPCFilterCombo.Add;
begin
  fFilterItems.Add(TFilterItem.Create);
end;

procedure TSPCFilterCombo.DeleteItem;
begin
  fFilterItems.Delete(Index);
end;

function TSPCFilterCombo.Count: Integer;
begin
  Result:=fFilterItems.Count;
end;

function TSPCFilterCombo.GetFilterItem;
begin
  Result:=fFilterItems.Items[Index];
end;

procedure TSPCFilterCombo.SetFilterItem;
begin
  fFilterItems.Items[Index]:=Value;
end;

procedure TSPCFilterCombo.DoChange(Obj: PObj);
begin
  Filter:=Items[fControl.CurIndex].Filter;
  if Assigned(fOnChange) then fOnChange(@Self);
end;

destructor TSPCFilterCombo.Destroy;
begin
   fFont.Free;
   inherited;
end;

function TSPCFilterCombo.SetPosition(X, Y: integer): PSPCFilterCombo;
begin
   fControl.Left := X;
   fControl.Top := Y;
   Result := @self;
end;

function TSPCFilterCombo.SetSize(X, Y: integer): PSPCFilterCombo;
begin
   fControl.Width  := X;
   fControl.Height := Y;
   Result := @self;
end;

function TSPCFilterCombo.GetFont;
begin
   Result := fFont;
end;

procedure TSPCFilterCombo.BuildList;
var
  fT: string;
  i: Integer;
begin
  fControl.Color:=Color;
  fControl.Font.Assign(Font);
  if Length(Text)>0 then
  begin
    fT:=Text;
    i:=0;
    Add;
    repeat
      if fT[1]<>#13 then
      begin
        Items[i].Full:=Items[i].Full+fT[1];
        Delete(fT,1,1);
      end else
      begin
        Add;
        Inc(i);
        Delete(fT,1,2);
      end;
    until Length(fT)=0;
    if Length(Items[Count-1].Full)=0 then DeleteItem(Count-1);
    for i:=0 to Count-1 do
    begin
      Items[i].Description:=Copy(Items[i].Full,1,Pos('|',Items[i].Full)-1);
      Items[i].Filter:=Copy(Items[i].Full,Pos('|',Items[i].Full)+1,Length(Items[i].Full)-Pos('|',Items[i].Full));
      fControl.Add(Items[i].Description);
    end;
  end;
end;

procedure TSPCFilterCombo.SetFilter(Value: string);
begin
  fFilter:=Value;
  if Assigned(fOnChange) then fOnChange(@Self);
end;

procedure TSPCFilterCombo.SetCurIndex(Value: Integer);
begin
  fCurIndex:=Value;
  fControl.CurIndex:=Value;
  Inc(fInitialized);
  if fInitialized>2 then fInitialized:=2;
  if Assigned(fOnChange) then if fInitialized=2 then fOnChange(@Self);
end;

end.
